#include <chronoptics/tof/csf_writer.hpp>
#include <chronoptics/tof/kea_camera.hpp>
#include <iostream>

namespace tof = chronoptics::tof;

int main() {
  // Filename to save to
  const char* filename = "frames_compare.csf";

  // Number of frames to capture
  const int numFrames = 10;

  // Number of frames to drop before capturing
  const int dropFrames = 10;

  const bool capture_raw = false;

  // Frame types to capture
  std::vector<tof::FrameType> types;

  if (capture_raw) {
    types = {tof::FrameType::RAW};
  } else {
    types = {tof::FrameType::RAW, tof::FrameType::RADIAL};
  }

  // Serial number of the camera to connect to. If this is empty, the
  // first camera that is found will be connected to
  const char* serial = "202001a";

  tof::KeaCamera cam(serial);

  if (capture_raw) {
    cam.set_on_camera_processing(false);
  }

  tof::select_streams(cam, types);

  tof::CsfWriter writer = tof::create_csf_writer_camera(filename, cam);

  cam.start();

  // Drop frames
  for (int i = 0; i < dropFrames; i++) {
    cam.get_frames();
  }

  std::cout << "Frames dropped" << std::endl;

  // Capture frames
  for (int i = 0; i < numFrames; i++) {
    std::vector<tof::Data> frames = cam.get_frames();

    for (tof::Data& frame : frames) {
      writer.write_frame(frame);
    }
  }

  std::cout << "Frames captured" << std::endl;
}
